# -*- coding: utf-8 -*-
#
#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import shutil
import os
import os.path as op
import sys
import imp
import tempfile

from time import gmtime, strftime
from CGNS.pyCGNSconfig import version as __vid__
from CGNS.pyCGNSconfig import HAS_MSW

WINOPTS = HAS_MSW

import CGNS.PAT.cgnskeywords as CGK

from qtpy.QtGui import QFont, QFontDatabase, QTextCursor

# User file paths:
conf_path = op.join(op.expanduser("~"), ".CGNS.NAV")
tmp_path = "/tmp"
if sys.platform == "win32":
    tmp_path = os.environ["TEMP"]


# -----------------------------------------------------------------
def removeSubDirAndFiles(path):
    shutil.rmtree(path)


def copyOneFile(src, dst):
    f1 = open(src, "r")
    l1 = f1.readlines()
    f1.close()
    if op.exists(dst):
        os.remove(dst)
    f2 = open(dst, "w")
    f2.writelines(l1)
    f2.close()


# -----------------------------------------------------------------
class Q7OptionContext(object):
    Q_VAR_NODE = "NODE"
    Q_VAR_PARENT = "PARENT"
    Q_VAR_NAME = "NAME"
    Q_VAR_VALUE = "VALUE"
    Q_VAR_CGNSTYPE = "SIDSTYPE"
    Q_VAR_CHILDREN = "CHILDREN"
    Q_VAR_TREE = "TREE"
    Q_VAR_PATH = "PATH"
    Q_VAR_LINKS = "LINKS"
    Q_VAR_SKIPS = "SKIPS"
    Q_VAR_RESULT = "RESULT"
    Q_VAR_USER = "ARGS"
    Q_VAR_SELECTED = "SELECTED"
    Q_VAR_RESULT_LIST = "__Q7_QUERY_RESULT__"

    Q_SCRIPT_PRE = """
import CGNS.PAT.cgnskeywords as CGK
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnslib as CGL
import numpy
"""

    Q_FILE_PRE = """
import CGNS.PAT.cgnskeywords as CGK
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnslib as CGL
import CGNS.NAV.moption as CGO
import numpy
"""

    Q_SCRIPT_POST = """
try:
  %s[0]=%s
except NameError:
  %s[0]=None
""" % (
        Q_VAR_RESULT_LIST,
        Q_VAR_RESULT,
        Q_VAR_RESULT_LIST,
    )

    _depends = {
        "MaxDisplayDataSize": ["DoNotDisplayLargeData"],
        "MaxLoadDataSize": ["DoNotLoadLargeArrays"],
        "MaxRecursionLevel": ["RecursiveTreeDisplay"],
    }
    _HasProPackage = True
    CHLoneTrace = False
    QueryNoException = False
    ActivateMultiThreading = False
    NAVTrace = False
    AutoExpand = False
    RecursiveTreeDisplay = False
    OneViewPerTreeNode = False
    ShowTableIndex = True
    RecursiveSIDSPatternsLoad = True
    DoNotDisplayLargeData = False
    CheckOnTheFly = False
    FollowLinksAtLoad = True
    DoNotFollowLinksAtSave = True
    AddCurrentDirInSearch = True
    AddRootDirInSearch = True
    DoNotLoadLargeArrays = True
    ShowSIDSStatusColumn = True
    ForceSIDSLegacyMapping = False
    ForceFortranFlag = True
    FilterCGNSFiles = True
    FilterHDFFiles = True
    FilterOwnFiles = True
    FileUpdateRemovesChildren = True
    TransposeArrayForView = True
    Show1DAsPlain = True
    ShowSIDSColumn = True
    ShowLinkColumn = True
    ShowSelectColumn = True
    ShowUserColumn = True
    ShowCheckColumn = True
    ShowShapeColumn = True
    ShowDataTypeColumn = True
    UserCSS = ""
    SelectionListDirectory = op.normpath(op.join(conf_path, "selections"))
    QueriesDirectory = op.normpath(op.join(conf_path, "queries"))
    FunctionsDirectory = op.normpath(op.join(conf_path, "functions"))
    SnapShotDirectory = op.normpath(op.join(conf_path, "snapshots"))
    _HistoryFileName = op.normpath(op.join(conf_path, "historyfile.py"))
    _OptionsFileName = op.normpath(op.join(conf_path, "optionsfile.py"))
    _QueriesDefaultFile = "default.py"
    _FunctionsDefaultFile = "default.py"
    IgnoredMessages = []
    LinkSearchPathList = []
    ProfileSearchPathList = []
    GrammarSearchPathList = []
    ValKeyList = ["sample"]
    CGNSFileExtension = [".cgns", ".adf"]
    HDFFileExtension = [".hdf", ".hdf5"]
    OwnFileExtension = [".cgh"]
    MaxLoadDataSize = 1000
    MaxDisplayDataSize = 1000
    MaxRecursionLevel = 7
    ADFConversionCom = "cgnsconvert"
    TemporaryDirectory = op.normpath(tmp_path)
    _ConvertADFFiles = True
    _ToolName = "CGNS.NAV"
    _ToolVersion = "%s" % __vid__

    _CopyrightNotice = """
Copyright (c) Marc Poinot <br>
Copyright (c) Safran - www.safran-group.com<br><br>
Copyright (c) Onera - www.onera.fr<br><br>
<b>all other copyrights and used versions listed below</b>

<hr>
<h3>Contributors (alphabetic order)</h3>
<table>
<tr><td>Florent Cayré</td><td>-SNECMA, France</td></tr>
<tr><td>Alexandre Fayolle</td><td>-LOGILAB, France</td></tr>
<tr><td>Xavier Garnaud</td><td>-SAFRAN, France</td></tr>
<tr><td>Loic Hauchard</td><td>-ONERA (Student INSA Rouen, France)</td></tr>
<tr><td>Elise Hénaux</td><td>-ONERA (Student FIIFO Orsay, France)</td></tr>
<tr><td>Grégory Laheurte</td><td>-ONERA, France</td></tr>
<tr><td>Pierre-Jacques Legay</td><td>-BERTIN, France</td></tr>
<tr><td>Bill Perkins</td><td>-Pacific Northwest Ntl Lab, U.S.A.</td></tr>
<tr><td>Mickael Philit</td><td>-SAFRAN, France</td></tr>
<tr><td>Jérôme Regis</td><td>-STILOG, France</td></tr>
<tr><td>Benoit Rodriguez</td><td>-ONERA, France</td></tr>
<tr><td>Tristan Soubrié</td><td>-ANDHEO, France</td></tr>
<tr><td>Francois Thirifays</td><td>-CENAERO, Belgique</td></tr>
<tr><td>Simon Verley</td><td>-ONERA, France</td></tr>
<tr><td>Ching-Yang Wang</td><td>-U.S.A.</td></tr>
</table>

<h2>Copyrights & Versions</h2>
<hr>
%(pycgnsversion)s<br>
All <b>pyCGNS</b> 
rights reserved in accordance with GPL v2 <br><br>
<b>NO WARRANTY :</b><br>
Check GPL v2 sections 15 and 16
about <font color=red>loss of data or corrupted data</font><br>
<hr>
%(pyqtversion)s<br>
PyQt Copyright (c) Riverbank Computing Limited. <br>
Usage within the terms of the GPL v2.<br>
All Rights Reserved.<br>
<hr>
%(qtversion)s<br>
Qt Copyright (c)<br>
The Qt4 Library is (c) 2011 Nokia Corporation and/or its subsidiary(-ies),
and is licensed under the GNU Lesser General Public License version 2.1
with Nokia Qt LGPL exception version 1.1. <br>
<hr>
%(pythonversion)s<br>
Python Copyright (c)<br>
Copyright (c) 2001-2011 Python Software Foundation.<br>
All Rights Reserved.<br>
<br>
Copyright (c) 2000 BeOpen.com.<br>
All Rights Reserved.<br>
<br>
Copyright (c) 1995-2001 Corporation for National Research Initiatives.<br>
All Rights Reserved.<br>
<br>
Copyright (c) 1991-1995 Stichting Mathematisch Centrum, Amsterdam.<br>
All Rights Reserved.<br>
<br>
<hr>
%(numpyversion)s<br>
Numpy Copyright (c)<br>
Copyright (c) 2005, NumPy Developers<br>
<hr>
%(hdf5version)s<br>
HDF5 Copyright (c)<br>
HDF5 (Hierarchical Data Format 5) Software Library and Utilities<br>
Copyright 2006-2013 by The HDF Group.<br>
NCSA HDF5 (Hierarchical Data Format 5) Software Library and Utilities<br>
Copyright 1998-2006 by the Board of Trustees of the University of Illinois.<br>
<hr>
%(vtkversion)s<br>
VTK Copyright (c)<br>
Copyright (c) 1993-2008 Ken Martin, Will Schroeder, Bill Lorensen <br>
All rights reserved.<br>
<hr>
%(cythonversion)s<br>
Cython Copyright (c)<br>
(c) Copyright 2012, Stefan Behnel, Robert Bradshaw, Dag Sverre Seljebotn,
Greg Ewing, William Stein, Gabriel Gellner, et al.
<hr>
Icons Copyright (c)<br>
All these nice icons are provided or are modified versions of original
icons provided by Mark James (Birmingham, UK).<br>
Please visit his web site: http://www.famfamfam.com/<br>

"""
    # __fv = QFont(QFontDatabase().families()[0]) # Spyder-like but needs a QApplication
    __fv = QFont("Courier new")
    __fc = QFont("Courier new")

    _Label_Font = QFont(__fv)
    _Button_Font = QFont(__fv)
    _Table_Font = QFont(__fc)
    _Edit_Font = QFont(__fc)
    _RName_Font = QFont(__fc)
    _NName_Font = QFont(__fc)
    _NName_Font.setBold(True)

    _Default_Fonts = {
        "Label_Family": _Label_Font.family(),
        "Label_Size": _Label_Font.pointSize(),
        "Label_Bold": False,
        "Label_Italic": False,
        "Table_Family": _Table_Font.family(),
        "Table_Size": _Table_Font.pointSize(),
        "Table_Bold": False,
        "Table_Italic": False,
        "Edit_Family": _Edit_Font.family(),
        "Edit_Size": _Edit_Font.pointSize(),
        "Edit_Bold": False,
        "Edit_Italic": False,
        "Button_Family": _Button_Font.family(),
        "Button_Size": _Button_Font.pointSize(),
        "Button_Bold": False,
        "Button_Italic": False,
        "RName_Family": _RName_Font.family(),
        "RName_Size": _RName_Font.pointSize(),
        "RName_Bold": False,
        "RName_Italic": False,
        "NName_Family": _NName_Font.family(),
        "NName_Size": _NName_Font.pointSize(),
        "NName_Bold": False,
        "NName_Italic": True,
    }

    UserColors = [
        "gray",
        "red",
        "green",
        "blue",
        "orange",
        None,
        None,
        None,
        None,
        None,
    ]

    _ColorList = {
        "cold_grey": (0.5000, 0.5400, 0.5300),
        "dim_grey": (0.4118, 0.4118, 0.4118),
        "grey": (0.7529, 0.7529, 0.7529),
        "light_grey": (0.8275, 0.8275, 0.8275),
        "slate_grey": (0.4392, 0.5020, 0.5647),
        "slate_grey_dark": (0.1843, 0.3098, 0.3098),
        "slate_grey_light": (0.4667, 0.5333, 0.6000),
        "warm_grey": (0.5000, 0.5000, 0.4100),
        "black": (0.0000, 0.0000, 0.0000),
        "ivory_black": (0.1600, 0.1400, 0.1300),
        "lamp_black": (0.1800, 0.2800, 0.2300),
        "alizarin_crimson": (0.8900, 0.1500, 0.2100),
        "brick": (0.6100, 0.4000, 0.1200),
        "coral": (1.0000, 0.4980, 0.3137),
        "coral_light": (0.9412, 0.5020, 0.5020),
        "deep_pink": (1.0000, 0.0784, 0.5765),
        "firebrick": (0.6980, 0.1333, 0.1333),
        "geranium_lake": (0.8900, 0.0700, 0.1900),
        "hot_pink": (1.0000, 0.4118, 0.7059),
        "light_salmon": (1.0000, 0.6275, 0.4784),
        "madder_lake_deep": (0.8900, 0.1800, 0.1900),
        "maroon": (0.6902, 0.1882, 0.3765),
        "pink": (1.0000, 0.7529, 0.7961),
        "pink_light": (1.0000, 0.7137, 0.7569),
        "raspberry": (0.5300, 0.1500, 0.3400),
        "rose_madder": (0.8900, 0.2100, 0.2200),
        "salmon": (0.9804, 0.5020, 0.4471),
        "tomato": (1.0000, 0.3882, 0.2784),
        "beige": (0.6400, 0.5800, 0.5000),
        "brown": (0.5000, 0.1647, 0.1647),
        "brown_madder": (0.8600, 0.1600, 0.1600),
        "brown_ochre": (0.5300, 0.2600, 0.1200),
        "burlywood": (0.8706, 0.7216, 0.5294),
        "burnt_sienna": (0.5400, 0.2100, 0.0600),
        "burnt_umber": (0.5400, 0.2000, 0.1400),
        "chocolate": (0.8235, 0.4118, 0.1176),
        "deep_ochre": (0.4500, 0.2400, 0.1000),
        "flesh": (1.0000, 0.4900, 0.2500),
        "flesh_ochre": (1.0000, 0.3400, 0.1300),
        "gold_ochre": (0.7800, 0.4700, 0.1500),
        "greenish_umber": (1.0000, 0.2400, 0.0500),
        "khaki": (0.9412, 0.9020, 0.5490),
        "khaki_dark": (0.7412, 0.7176, 0.4196),
        "light_beige": (0.9608, 0.9608, 0.8627),
        "peru": (0.8039, 0.5216, 0.2471),
        "rosy_brown": (0.7373, 0.5608, 0.5608),
        "raw_sienna": (0.7800, 0.3800, 0.0800),
        "raw_umber": (0.4500, 0.2900, 0.0700),
        "sepia": (0.3700, 0.1500, 0.0700),
        "sienna": (0.6275, 0.3216, 0.1765),
        "saddle_brown": (0.5451, 0.2706, 0.0745),
        "sandy_brown": (0.9569, 0.6431, 0.3765),
        "tan": (0.8235, 0.7059, 0.5490),
        "van_dyke_brown": (0.3700, 0.1500, 0.0200),
        "cadmium_orange": (1.0000, 0.3800, 0.0100),
        "carrot": (0.9300, 0.5700, 0.1300),
        "dark_orange": (1.0000, 0.5490, 0.0000),
        "mars_orange": (0.5900, 0.2700, 0.0800),
        "mars_yellow": (0.8900, 0.4400, 0.1000),
        "orange": (1.0000, 0.5000, 0.0000),
        "orange_red": (1.0000, 0.2706, 0.0000),
        "yellow_ochre": (0.8900, 0.5100, 0.0900),
        "aureoline_yellow": (1.0000, 0.6600, 0.1400),
        "banana": (0.8900, 0.8100, 0.3400),
        "cadmium_lemon": (1.0000, 0.8900, 0.0100),
        "cadmium_yellow": (1.0000, 0.6000, 0.0700),
        "cadmium_yellow_light": (1.0000, 0.6900, 0.0600),
        "gold": (1.0000, 0.8431, 0.0000),
        "goldenrod": (0.8549, 0.6471, 0.1255),
        "goldenrod_dark": (0.7216, 0.5255, 0.0431),
        "goldenrod_light": (0.9804, 0.9804, 0.8235),
        "goldenrod_pale": (0.9333, 0.9098, 0.6667),
        "light_goldenrod": (0.9333, 0.8667, 0.5098),
        "melon": (0.8900, 0.6600, 0.4100),
        "naples_yellow_deep": (1.0000, 0.6600, 0.0700),
        "yellow": (1.0000, 1.0000, 0.0000),
        "yellow_light": (1.0000, 1.0000, 0.8784),
        "chartreuse": (0.4980, 1.0000, 0.0000),
        "chrome_oxide_green": (0.4000, 0.5000, 0.0800),
        "cinnabar_green": (0.3800, 0.7000, 0.1600),
        "cobalt_green": (0.2400, 0.5700, 0.2500),
        "emerald_green": (0.0000, 0.7900, 0.3400),
        "forest_green": (0.1333, 0.5451, 0.1333),
        "green": (0.0000, 1.0000, 0.0000),
        "green_dark": (0.0000, 0.3922, 0.0000),
        "green_pale": (0.5961, 0.9843, 0.5961),
        "green_yellow": (0.6784, 1.0000, 0.1843),
        "lawn_green": (0.4863, 0.9882, 0.0000),
        "lime_green": (0.1961, 0.8039, 0.1961),
        "mint": (0.7400, 0.9900, 0.7900),
        "olive": (0.2300, 0.3700, 0.1700),
        "olive_drab": (0.4196, 0.5569, 0.1373),
        "olive_green_dark": (0.3333, 0.4196, 0.1843),
        "permanent_green": (0.0400, 0.7900, 0.1700),
        "sap_green": (0.1900, 0.5000, 0.0800),
        "sea_green": (0.1804, 0.5451, 0.3412),
        "sea_green_dark": (0.5608, 0.7373, 0.5608),
        "sea_green_medium": (0.2353, 0.7020, 0.4431),
        "sea_green_light": (0.1255, 0.6980, 0.6667),
        "spring_green": (0.0000, 1.0000, 0.4980),
        "spring_green_medium": (0.0000, 0.9804, 0.6039),
        "terre_verte": (0.2200, 0.3700, 0.0600),
        "viridian_light": (0.4300, 1.0000, 0.4400),
        "yellow_green": (0.6039, 0.8039, 0.1961),
        "aquamarine": (0.4980, 1.0000, 0.8314),
        "aquamarine_medium": (0.4000, 0.8039, 0.6667),
        "cyan": (0.0000, 1.0000, 1.0000),
        "cyan_white": (0.8784, 1.0000, 1.0000),
        "turquoise": (0.2510, 0.8784, 0.8157),
        "turquoise_dark": (0.0000, 0.8078, 0.8196),
        "turquoise_medium": (0.2824, 0.8196, 0.8000),
        "turquoise_pale": (0.6863, 0.9333, 0.9333),
        "alice_blue": (0.9412, 0.9725, 1.0000),
        "blue": (0.0000, 0.0000, 1.0000),
        "blue_light": (0.6784, 0.8471, 0.9020),
        "blue_medium": (0.0000, 0.0000, 0.8039),
        "cadet": (0.3725, 0.6196, 0.6275),
        "cobalt": (0.2400, 0.3500, 0.6700),
        "cornflower": (0.3922, 0.5843, 0.9294),
        "cerulean": (0.0200, 0.7200, 0.8000),
        "dodger_blue": (0.1176, 0.5647, 1.0000),
        "indigo": (0.0300, 0.1800, 0.3300),
        "manganese_blue": (0.0100, 0.6600, 0.6200),
        "midnight_blue": (0.0980, 0.0980, 0.4392),
        "navy": (0.0000, 0.0000, 0.5020),
        "peacock": (0.2000, 0.6300, 0.7900),
        "powder_blue": (0.6902, 0.8784, 0.9020),
        "royal_blue": (0.2549, 0.4118, 0.8824),
        "slate_blue": (0.4157, 0.3529, 0.8039),
        "slate_blue_dark": (0.2824, 0.2392, 0.5451),
        "slate_blue_light": (0.5176, 0.4392, 1.0000),
        "slate_blue_medium": (0.4824, 0.4078, 0.9333),
        "sky_blue": (0.5294, 0.8078, 0.9216),
        "sky_blue_deep": (0.0000, 0.7490, 1.0000),
        "sky_blue_light": (0.5294, 0.8078, 0.9804),
        "steel_blue": (0.2745, 0.5098, 0.7059),
        "steel_blue_light": (0.6902, 0.7686, 0.8706),
        "turquoise_blue": (0.0000, 0.7800, 0.5500),
        "ultramarine": (0.0700, 0.0400, 0.5600),
        "blue_violet": (0.5412, 0.1686, 0.8863),
        "cobalt_violet_deep": (0.5700, 0.1300, 0.6200),
        "magenta": (1.0000, 0.0000, 1.0000),
        "orchid": (0.8549, 0.4392, 0.8392),
        "orchid_dark": (0.6000, 0.1961, 0.8000),
        "orchid_medium": (0.7294, 0.3333, 0.8275),
        "permanent_red_violet": (0.8600, 0.1500, 0.2700),
        "plum": (0.8667, 0.6275, 0.8667),
        "purple": (0.6275, 0.1255, 0.9412),
        "purple_medium": (0.5765, 0.4392, 0.8588),
        "ultramarine_violet": (0.3600, 0.1400, 0.4300),
        "violet": (0.5600, 0.3700, 0.6000),
        "violet_dark": (0.5804, 0.0000, 0.8275),
        "violet_red": (0.8157, 0.1255, 0.5647),
        "violet_red_medium": (0.7804, 0.0824, 0.5216),
        "violet_red_pale": (0.8588, 0.4392, 0.5765),
    }

    _ReservedNames = CGK.cgnsnames
    _ReservedTypes = CGK.cgnstypes
    _SortedTypeList = CGK.sortedtypelist

    _UsualQueries = [
        # ---------------------------------------------------------------------------
        # INDENTATION IS SIGNIFICANT
        # ---------------------------------------------------------------------------
        # --- Search -----------------------------------------------------------
        # last two booleans: Update tree, has args
        (
            "001. Node name",
            "Search by",
            "RESULT=(NAME==ARGS[0])",
            """
Search by
Node name

Search all nodes with the exact NAME as argument.

The argument name need not to be a tuple or to have quotes,
all the following values are ok and would match the NAME <i>ZoneType</i>:

ZoneType
'ZoneType'
('ZoneType',)
""",
            False,
            True,
        ),
        (
            "002. Wildcard node name",
            "Search by",
            """import fnmatch
RESULT=fnmatch.fnmatchcase(NAME,ARGS[0])
""",
            """
Search by
Wildcard node name

Search all nodes with the wildcard NAME as argument.

Warning: the <b>argument name</b> should be quoted:

'BC*' is ok

BC* would fail
""",
            False,
            True,
        ),
        (
            "003. Node type",
            "Search by",
            "RESULT=(SIDSTYPE==ARGS[0])",
            """search all nodes with argument SIDS type.""",
            False,
            True,
        ),
        (
            "005. Node value",
            "Search by",
            """
from numpy import *
target=eval(ARGS[0])
if   (VALUE is None and target is None): RESULT=True
elif (VALUE is None)            : RESULT=False
elif (target.dtype!=VALUE.dtype): RESULT=False
elif (target.size!=VALUE.size):   RESULT=False
elif (target.shape!=VALUE.shape): RESULT=False
elif (target.tolist()==VALUE.tolist()): RESULT=True
else:                             RESULT=False
""",
            """search all nodes with argument value. The compare is performed
using a straightforward '==' and then relies on the Python/Numpy comparison
operator.""",
            False,
            True,
        ),
        (
            "010. Node with truncated data",
            "Search by",
            "if (PATH in SKIPS): RESULT=PATH",
            """search all nodes with truncated or unread data, for example if you have set
the maximum data argument for the load, or if you release the memory of a
node.""",
            False,
            False,
        ),
        (
            "004. Wildcard node type",
            "Search by",
            """
import fnmatch
RESULT=fnmatch.fnmatchcase(SIDSTYPE,ARGS[0])
""",
            """
Search by
Wildcard node type

Search all nodes with wildcard argument SIDS type.
Warning: the <b>argument type</b> should be quoted:

'Turb*' is ok

Turb* would fail
""",
            False,
            True,
        ),
        (
            "011. Non-MT UserDefinedData",
            "Search by",
            """
if (    (SIDSTYPE==CGK.UserDefinedData_ts)
    and (CGU.getValueDataType(NODE)!=CGK.MT)):
  RESULT=True
""",
            """
Search by
Valued UserDefinedData

Search all <b>UserDefinedData_t</b> nodes with a non-<b>MT</b> data type.

No argument.
""",
            False,
            False,
        ),
        # -----------------------------------------------------------------------------
        (
            "012. FamilyName",
            "Search by",
            """
if (SIDSTYPE in [CGK.FamilyName_ts, CGK.AdditionalFamilyName_ts]):
    RESULT=True
""",
            """
Search by
All <b>FamilyName_t</b> and <b>AdditionalFamilyname_t</b> nodes.
""",
            False,
            False,
        ),
        # -----------------------------------------------------------------------------
        (
            "013. FamilyName reference",
            "Search by",
            """
if ((SIDSTYPE in [CGK.FamilyName_ts, CGK.AdditionalFamilyName_ts]) and
    (VALUE.tostring().decode('ascii')==ARGS[0])):
    RESULT=True
""",
            """
Search by<br>
Reference to a FamilyName<br>
 
Search all <b>FamilyName</b> nodes with the arg string (plain).<br>
The string arg should be a valid Python string such as:<br>
 
'BLADE(1)'<br>
'Surface ext 1A'<br>
'Left/Wing/Flap'<br>
 
""",
            False,
            True,
        ),
        # -----------------------------------------------------------------------------
        (
            "014. Zones",
            "Search by",
            """
if (SIDSTYPE in [CGK.Zone_ts]):
    RESULT=True
""",
            """
Search by
All <b>Zone_t</b> nodes.
""",
            False,
            False,
        ),
        # -----------------------------------------------------------------------------
        (
            "015. Zones Structured",
            "Search by",
            """
if (SIDSTYPE in [CGK.Zone_ts]):
    t=CGU.hasChildName(NODE,CGK.ZoneType_s)
    if (t is None or CGU.stringValueMatches(t,CGK.Structured_s)):
      RESULT=True
""",
            """
Search by
All <b>Zone_t</b> with Structured <b>ZoneType</b> nodes.
""",
            False,
            False,
        ),
        # -----------------------------------------------------------------------------
        (
            "016. Zones Unstructured",
            "Search by",
            """
if (SIDSTYPE in [CGK.Zone_ts]):
    t=CGU.hasChildName(NODE,CGK.ZoneType_s)
    if (t is not None and CGU.stringValueMatches(t,CGK.Unstructured_s)):
      RESULT=True
""",
            """
Search by
All <b>Zone_t</b> with Unstructured <b>ZoneType</b> nodes.
""",
            False,
            False,
        ),
        # -----------------------------------------------------------------------------
        (
            "017. BCs",
            "Search by",
            """
if (SIDSTYPE in [CGK.BC_ts]):
    RESULT=True
""",
            """
Search by
All <b>BC_t</b> nodes.
""",
            False,
            False,
        ),
        # --- Replace
        # -----------------------------------------------------------------------------
        (
            "050. Valued UserDefinedData",
            "Replace",
            """
if (     (SIDSTYPE==CGK.UserDefinedData_ts)
     and (CGU.getValueDataType(NODE)!=CGK.MT)):
  NODE[3]=CGK.DataArray_ts
  RESULT=True
""",
            """
Replace
Valued UserDefinedData
 
Search all <b>UserDefinedData_t</b> nodes with a non-<b>MT</b> data type
and replace them as <b>DataArray_t</b>.""",
            False,
            False,
        ),
        (
            "051. Substitute Zone name",
            "Replace",
            """
l1=len(ARGS[0])
if ((SIDSTYPE==CGK.Zone_ts) and (NAME[:l1]==ARGS[0])):
  NODE[0]=ARGS[1]+NODE[0][l1:]
  RESULT=True

if (CGU.getValueDataType(NODE)==CGK.C1):
  v=VALUE.tostring().decode('ascii')
  if (v[:l1]==ARGS[0]):
    v=ARGS[1]+v[l1:]
    NODE[1]=CGU.setStringAsArray(v)
    RESULT=True
""",
            """
<h1>Replace</h1>
<h2>Substitute Zone name</h2>
<p>
Search all <b>Zone_t</b> nodes with a name pattern, then rename the
zone with the substitution pattern. Any other reference in the tree,
as a connectivity value for example, is subsitued as well.
<p>
Argument is a tuple with the first pattern to find and
the second as the subsitution pattern. For example:
<pre>
('domain.','zone#')
</pre>
""",
            True,
            True,
        ),
        # -----------------------------------------------------------------------------
        (
            "052. FamilySpecified BC type rewrite",
            "Replace",
            """
if ((SIDSTYPE in [CGK.FamilyName_ts, CGK.AdditionalFamilyName_ts]) and
    (VALUE.tostring().decode('ascii')==ARGS[0]) and (PARENT[3]==CGK.BC_ts)):
    PARENT[1]=CGU.setStringAsArray(CGK.FamilySpecified_s)
    RESULT=True
""",
            """
<h1>Replace</h1>
<h2>FamilySpecified BC type rewrite</h2>
<p>
Search all <b>FamilyName BC</b> nodes with the arg string (plain).<br>
The string arg should be a valid Python string such as:<br>

'BLADE(1)'<br>
'Surface ext 1A'<br>
'Left/Wing/Flap'<br>

<p>
Once found, the parent <b>BC_t</b> value is forced to <b>FamilySpecified</b>

""",
            True,
            True,
        ),
        # --- Find Elements_t
        (
            "020. Elements",
            "Find Elements_t",
            """if (SIDSTYPE==CGK.Elements_ts):
  RESULT=True
""",
            """Find all <b>Elements_t</b> nodes """,
            False,
            False,
        ),
        (
            "021. Elements QUAD",
            "Find Elements_t",
            """if (SIDSTYPE==CGK.Elements_ts):
  RESULT=VALUE[0] in (CGK.QUAD_4, CGK.QUAD_8, CGK.QUAD_9)
""",
            """Find all <b>Elements_t</b> nodes of type <b>QUAD</b>""",
            False,
            False,
        ),
        (
            "022. Elements TRI",
            "Find Elements_t",
            """if (SIDSTYPE==CGK.Elements_ts):
  RESULT=VALUE[0] in (CGK.TRI_3, CGK.TRI_6)
""",
            """Find all <b>Elements_t</b> nodes of type <b>TRI</b>""",
            False,
            False,
        ),
        (
            "023. Elements NGON",
            "Find Elements_t",
            """if (SIDSTYPE==CGK.Elements_ts):
  RESULT=VALUE[0] in (CGK.NGON_n,)
""",
            """Find all <b>Elements_t</b> nodes of type <b>NGON_n</b>""",
            False,
            False,
        ),
        (
            "024. Elements HEXA",
            "Find Elements_t",
            """if (SIDSTYPE==CGK.Elements_ts):
  RESULT=VALUE[0] in (CGK.HEXA_8, CGK.HEXA_20, CGK.HEXA_27)
""",
            """Find all <b>Elements_t</b> nodes of type <b>HEXA</b>""",
            False,
            False,
        ),
        (
            "025. Elements TETRA",
            "Find Elements_t",
            """if (SIDSTYPE==CGK.Elements_ts):
  RESULT=VALUE[0] in (CGK.TETRA_4, CGK.TETRA_10)
""",
            """Find all <b>Elements_t</b> nodes of type <b>TETRA</b>""",
            False,
            False,
        ),
        # --- External Tools
        (
            "030. Create Cartesian Zone",
            "External Tools",
            """
if (SIDSTYPE==CGK.CGNSTree_ts):
    import Generator.PyTree as G
    z=G.cart((0.,0.,0.), (0.1,0.1,0.2), (10,11,12))
    b=None
    base='BASE'
    if (len(ARGS)>0):
      base=ARGS[0]
      b=CGU.hasChildName(NODE,base)
    if (b is None):
      base=CGU.checkUniqueChildName(NODE,base)
      b=CGL.newCGNSBase(NODE,base,3,3)
    CGU.addChild(b,z)
""",
            """Example of Cartesian zone creation using Cassiopee.
The first argument is the base name, if ommitted a name is generated.""",
            True,
            True,
        ),
        (
            "031. Bounding boxes",
            "External Tools",
            """
if (SIDSTYPE==CGK.Zone_ts):
    import Generator as G
    RESULT=G.bbox(NODE)
""",
            """Example of Bounding box computation using Cassiopee""",
        ),
        (
            "100. .Solver#Compute children",
            "Edit filters",
            """
if (PARENT[0]=='.Solver#Compute'):
    RESULT=PATH
""",
            """Selects all children nodes of the .Solver#Compute elsA userdefined node""",
            False,
            False,
        ),
        (
            "101. ReferenceState children",
            "Edit filters",
            """
if (PARENT[0]=='ReferenceState'):
    RESULT=PATH
""",
            """Selects all children nodes of the ReferenceState node""",
            False,
            False,
        ),
        (
            "102. .Solver#Param children",
            "Edit filters",
            """
if (PARENT[0]=='.Solver#Param'):
   RESULT=PATH
""",
            """Selects all children nodes of the .Solver#Param elsA userdefined node""",
            False,
            False,
        ),
    ]

    # -----------------------------------------------------------------
    @classmethod
    def _setOption(cls, name, value):
        setattr(cls, name, value)

    @classmethod
    def _writeFile(cls, tag, name, udata, filename, prefix=""):
        gdate = strftime("%Y-%m-%d %H:%M:%S", gmtime())
        s = """# %s - %s - Generated %s\n# coding: utf-8\n%s\nimport PyQt5\n%s=""" % (
            cls._ToolName,
            tag,
            gdate,
            prefix,
            name,
        )
        if isinstance(udata, dict):
            s += """{\n"""
            for k in udata:
                #          print 'K',k,'V',udata[k],'T',type(udata[k])
                if k[0] != "_":
                    val = "%s" % str(udata[k])
                    if isinstance(udata[k], str):
                        val = 'u"""%s"""' % repr(udata[k]).lstrip("u'").lstrip(
                            "'"
                        ).rstrip(
                            "'"
                        )  # Error
                    elif isinstance(udata[k], bytes):
                        val = 'u"""%s"""' % repr(udata[k].decode("utf-8")).lstrip(
                            "u'"
                        ).lstrip("'").rstrip("'")
                    if not isinstance(k, str):
                        uk = "u'%s'" % repr(k.decode("utf-8")).lstrip("u'").lstrip(
                            "'"
                        ).rstrip("'")
                    else:
                        uk = "u'%s'" % repr(k).lstrip("u'").lstrip("'").rstrip("'")
                    s += """%s:%s,\n""" % (uk, val)
            s += """}\n\n# --- last line\n"""
        elif isinstance(udata, list):
            s += """[\n"""
            for k in udata:
                s += """%s,\n""" % (k)
            s += """]\n\n# --- last line\n"""
        cls._crpath(filename)
        with open(filename, "w+") as f:
            f.write(s)

    @classmethod
    def _readFile(cls, name, filename):
        dpath = tempfile.mkdtemp()
        if not op.exists(filename):
            return None
        try:
            copyOneFile(filename, "%s/%s.py" % (dpath, name))
        except IOError:
            removeSubDirAndFiles(dpath)
            return None
        sprev = sys.path
        sys.path = [dpath] + sys.path
        try:
            fp, pathname, description = imp.find_module(name)
        except IndexError:  # ImportError:
            return None
        try:
            mod = imp.load_module(name, fp, pathname, description)
        finally:
            if fp:
                fp.close()
        removeSubDirAndFiles(dpath)
        sys.path = sprev
        return mod

    @classmethod
    def _crpath(cls, path):
        p = op.dirname(path)
        if op.exists(p):
            return True
        os.makedirs(p)

    @classmethod
    def _trpath(cls, path):
        return op.normpath(op.expanduser(op.expandvars(path)))

    @classmethod
    def _writeHistory(cls, control):
        filename = cls._trpath(cls._HistoryFileName)
        cls._writeFile("History", "history", control._history, filename)

    @classmethod
    def _readHistory(cls, control):
        filename = cls._trpath(cls._HistoryFileName)
        m = cls._readFile("history", filename)
        if m is None:
            return None
        try:
            return m.history
        except:
            return None

    @classmethod
    def _writeOptions(cls, control):
        filename = cls._trpath(cls._OptionsFileName)
        cls._writeFile("User options", "options", control._options, filename)

    @classmethod
    def _readOptions(cls, control):
        filename = cls._trpath(cls._OptionsFileName)
        m = cls._readFile("options", filename)
        if m is None:
            return None
        try:
            return m.options
        except:
            return None

    @classmethod
    def _writeQueries(cls, control, q):
        filename = cls._trpath(op.join(cls.QueriesDirectory, cls._QueriesDefaultFile))
        cls._writeFile("User queries", "queries", q, filename, cls.Q_FILE_PRE)

    @classmethod
    def _readQueries(cls, control):
        filename = cls._trpath(op.join(cls.QueriesDirectory, cls._QueriesDefaultFile))
        m = cls._readFile("queries", filename)
        if m is None:
            return None
        try:
            return m.queries
        except:
            return None

    @classmethod
    def _writeFunctions(cls, control, f):
        filename = cls._trpath(
            op.join(cls.FunctionsDirectory, cls._FunctionsDefaultFile)
        )
        cls._writeFile("User functions", "functions", f, filename, cls.Q_FILE_PRE)

    @classmethod
    def _readFunctions(cls, control):
        filename = cls._trpath(
            op.join(cls.FunctionsDirectory, cls._FunctionsDefaultFile)
        )
        m = cls._readFile("functions", filename)
        if m is None:
            return None
        try:
            return m.Q7UserFunction
        except:
            return None

    def __init__(self):
        pass

    def __getitem__(self, name):
        if name[0] != "_":
            return Q7OptionContext.__dict__[name]
        return None

    def __setitem__(self, name, value):
        if name[0] != "_":
            setattr(Q7OptionContext, name, value)
        return None

    def __iter__(self):
        for o in dir(self):
            if o[0] != "_":
                yield o

    def _nextName(self):
        for o in dir(self):
            if o[0] != "_":
                yield o


# -----------------------------------------------------------------



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'CGNS/NAV/T/Q7LinkWindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Q7LinkWindow(object):
    def setupUi(self, Q7LinkWindow):
        Q7LinkWindow.setObjectName("Q7LinkWindow")
        Q7LinkWindow.resize(715, 350)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Q7LinkWindow.sizePolicy().hasHeightForWidth())
        Q7LinkWindow.setSizePolicy(sizePolicy)
        Q7LinkWindow.setMinimumSize(QtCore.QSize(715, 350))
        Q7LinkWindow.setMaximumSize(QtCore.QSize(3000, 750))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/icons/cgSpy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Q7LinkWindow.setWindowIcon(icon)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Q7LinkWindow)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.bCheckLink = QtWidgets.QPushButton(Q7LinkWindow)
        self.bCheckLink.setEnabled(True)
        self.bCheckLink.setMinimumSize(QtCore.QSize(25, 25))
        self.bCheckLink.setMaximumSize(QtCore.QSize(25, 25))
        self.bCheckLink.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/icons/link-check.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bCheckLink.setIcon(icon1)
        self.bCheckLink.setObjectName("bCheckLink")
        self.horizontalLayout.addWidget(self.bCheckLink)
        self.bLoadTree = QtWidgets.QPushButton(Q7LinkWindow)
        self.bLoadTree.setMinimumSize(QtCore.QSize(25, 25))
        self.bLoadTree.setMaximumSize(QtCore.QSize(25, 25))
        self.bLoadTree.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/images/icons/tree-load-g.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bLoadTree.setIcon(icon2)
        self.bLoadTree.setObjectName("bLoadTree")
        self.horizontalLayout.addWidget(self.bLoadTree)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.bAddLink = QtWidgets.QPushButton(Q7LinkWindow)
        self.bAddLink.setMinimumSize(QtCore.QSize(25, 25))
        self.bAddLink.setMaximumSize(QtCore.QSize(25, 25))
        self.bAddLink.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/images/icons/link-add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bAddLink.setIcon(icon3)
        self.bAddLink.setObjectName("bAddLink")
        self.horizontalLayout.addWidget(self.bAddLink)
        self.bDuplicateLink = QtWidgets.QPushButton(Q7LinkWindow)
        self.bDuplicateLink.setMinimumSize(QtCore.QSize(25, 25))
        self.bDuplicateLink.setMaximumSize(QtCore.QSize(25, 25))
        self.bDuplicateLink.setText("")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/images/icons/link-dup.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bDuplicateLink.setIcon(icon4)
        self.bDuplicateLink.setObjectName("bDuplicateLink")
        self.horizontalLayout.addWidget(self.bDuplicateLink)
        self.bFromSelection = QtWidgets.QPushButton(Q7LinkWindow)
        self.bFromSelection.setMinimumSize(QtCore.QSize(25, 25))
        self.bFromSelection.setMaximumSize(QtCore.QSize(25, 25))
        self.bFromSelection.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/images/icons/link-slist.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bFromSelection.setIcon(icon5)
        self.bFromSelection.setObjectName("bFromSelection")
        self.horizontalLayout.addWidget(self.bFromSelection)
        self.bDeleteLink = QtWidgets.QPushButton(Q7LinkWindow)
        self.bDeleteLink.setMinimumSize(QtCore.QSize(25, 25))
        self.bDeleteLink.setMaximumSize(QtCore.QSize(25, 25))
        self.bDeleteLink.setText("")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/images/icons/link-delete.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bDeleteLink.setIcon(icon6)
        self.bDeleteLink.setObjectName("bDeleteLink")
        self.horizontalLayout.addWidget(self.bDeleteLink)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.bSave = QtWidgets.QPushButton(Q7LinkWindow)
        self.bSave.setMinimumSize(QtCore.QSize(25, 25))
        self.bSave.setMaximumSize(QtCore.QSize(25, 25))
        self.bSave.setText("")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/images/icons/link-save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSave.setIcon(icon7)
        self.bSave.setObjectName("bSave")
        self.horizontalLayout.addWidget(self.bSave)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.line = QtWidgets.QFrame(Q7LinkWindow)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_2 = QtWidgets.QLabel(Q7LinkWindow)
        self.label_2.setEnabled(True)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_4.addWidget(self.label_2)
        self.cUnreachable = QtWidgets.QCheckBox(Q7LinkWindow)
        self.cUnreachable.setEnabled(True)
        self.cUnreachable.setObjectName("cUnreachable")
        self.horizontalLayout_4.addWidget(self.cUnreachable)
        self.cDuplicates = QtWidgets.QCheckBox(Q7LinkWindow)
        self.cDuplicates.setEnabled(True)
        self.cDuplicates.setObjectName("cDuplicates")
        self.horizontalLayout_4.addWidget(self.cDuplicates)
        self.cBad = QtWidgets.QCheckBox(Q7LinkWindow)
        self.cBad.setEnabled(True)
        self.cBad.setObjectName("cBad")
        self.horizontalLayout_4.addWidget(self.cBad)
        self.cExternal = QtWidgets.QCheckBox(Q7LinkWindow)
        self.cExternal.setEnabled(True)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(255, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Text, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.NoRole, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Text, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.NoRole, brush)
        brush = QtGui.QBrush(QtGui.QColor(118, 116, 113))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(118, 116, 113))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Text, brush)
        brush = QtGui.QBrush(QtGui.QColor(118, 116, 113))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.NoRole, brush)
        self.cExternal.setPalette(palette)
        font = QtGui.QFont()
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        font.setStrikeOut(False)
        self.cExternal.setFont(font)
        self.cExternal.setObjectName("cExternal")
        self.horizontalLayout_4.addWidget(self.cExternal)
        self.cInternal = QtWidgets.QCheckBox(Q7LinkWindow)
        self.cInternal.setEnabled(True)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(255, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(118, 116, 113))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.WindowText, brush)
        self.cInternal.setPalette(palette)
        self.cInternal.setObjectName("cInternal")
        self.horizontalLayout_4.addWidget(self.cInternal)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.cApplyToAll = QtWidgets.QCheckBox(Q7LinkWindow)
        self.cApplyToAll.setEnabled(True)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/images/icons/user-G.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cApplyToAll.setIcon(icon8)
        self.cApplyToAll.setObjectName("cApplyToAll")
        self.horizontalLayout_4.addWidget(self.cApplyToAll)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.linkTable = QtWidgets.QTableWidget(Q7LinkWindow)
        self.linkTable.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.linkTable.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.linkTable.setEditTriggers(QtWidgets.QAbstractItemView.DoubleClicked)
        self.linkTable.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.linkTable.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.linkTable.setColumnCount(5)
        self.linkTable.setObjectName("linkTable")
        self.linkTable.setRowCount(0)
        self.verticalLayout.addWidget(self.linkTable)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.bBackControl = QtWidgets.QPushButton(Q7LinkWindow)
        self.bBackControl.setMinimumSize(QtCore.QSize(25, 25))
        self.bBackControl.setMaximumSize(QtCore.QSize(25, 25))
        self.bBackControl.setText("")
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/images/icons/top.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bBackControl.setIcon(icon9)
        self.bBackControl.setObjectName("bBackControl")
        self.horizontalLayout_2.addWidget(self.bBackControl)
        self.bInfo = QtWidgets.QPushButton(Q7LinkWindow)
        self.bInfo.setMinimumSize(QtCore.QSize(25, 25))
        self.bInfo.setMaximumSize(QtCore.QSize(25, 25))
        self.bInfo.setText("")
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/images/icons/help-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bInfo.setIcon(icon10)
        self.bInfo.setObjectName("bInfo")
        self.horizontalLayout_2.addWidget(self.bInfo)
        self.eDirSource = QtWidgets.QLineEdit(Q7LinkWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.eDirSource.sizePolicy().hasHeightForWidth())
        self.eDirSource.setSizePolicy(sizePolicy)
        self.eDirSource.setObjectName("eDirSource")
        self.horizontalLayout_2.addWidget(self.eDirSource)
        self.label = QtWidgets.QLabel(Q7LinkWindow)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.eFileSource = QtWidgets.QLineEdit(Q7LinkWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.eFileSource.sizePolicy().hasHeightForWidth())
        self.eFileSource.setSizePolicy(sizePolicy)
        self.eFileSource.setObjectName("eFileSource")
        self.horizontalLayout_2.addWidget(self.eFileSource)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.bClose = QtWidgets.QPushButton(Q7LinkWindow)
        self.bClose.setObjectName("bClose")
        self.horizontalLayout_2.addWidget(self.bClose)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.retranslateUi(Q7LinkWindow)
        QtCore.QMetaObject.connectSlotsByName(Q7LinkWindow)

    def retranslateUi(self, Q7LinkWindow):
        _translate = QtCore.QCoreApplication.translate
        Q7LinkWindow.setWindowTitle(_translate("Q7LinkWindow", "Form"))
        self.bCheckLink.setToolTip(_translate("Q7LinkWindow", "check and fix link list"))
        self.bLoadTree.setToolTip(_translate("Q7LinkWindow", "open link target file"))
        self.bAddLink.setToolTip(_translate("Q7LinkWindow", "add new link entry"))
        self.bDuplicateLink.setToolTip(_translate("Q7LinkWindow", "duplicate link entry"))
        self.bFromSelection.setToolTip(_translate("Q7LinkWindow", "insert link from selection"))
        self.bDeleteLink.setToolTip(_translate("Q7LinkWindow", "delete link entry"))
        self.label_2.setText(_translate("Q7LinkWindow", "Select"))
        self.cUnreachable.setText(_translate("Q7LinkWindow", "unreachable"))
        self.cDuplicates.setText(_translate("Q7LinkWindow", "duplicates"))
        self.cBad.setText(_translate("Q7LinkWindow", "bad links"))
        self.cExternal.setText(_translate("Q7LinkWindow", "external links"))
        self.cInternal.setText(_translate("Q7LinkWindow", "internal links"))
        self.cApplyToAll.setText(_translate("Q7LinkWindow", "Apply to All Selected"))
        self.linkTable.setSortingEnabled(True)
        self.label.setText(_translate("Q7LinkWindow", "/"))
        self.bClose.setText(_translate("Q7LinkWindow", "Close"))
from . import Res_rc



#!/usr/bin/env python
# -------------------------------------------------------------------------
# pyCGNS.PAT - CFD General Notation System -
# See license.txt file in the root directory of this Python module source
# -------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU

T = CGL.newCGNSTree()
B = CGL.newBase(T, "Base", 3, 3)
Z = CGL.newZone(B, "Zone1")
Z = CGL.newZone(B, "Zone2")
Z = CGL.newZone(B, "Zone3")
Z = CGL.newZone(B, "Zone4")

l1 = CGU.getAllNodesByTypeList(T, ["CGNSTree_t", "CGNSBase_t", "Zone_t"])
l2 = CGU.getAllNodesByTypeSet(T, ["Zone_t", "IndexArray_t"])

print(l1)
print(l2)

print(CGU.getNodeByPath(l1[0], T))

l3 = CGU.getAllNodesByTypeList(B, ["CGNSBase_t", "Zone_t"])
l4 = CGU.getAllNodesByTypeSet(B, ["Zone_t"])

print(l3)
print(l4)

print(CGU.getNodeByPath(B, l3[0]))



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as CGL
import CGNS.MAP as CGM
from CGNS.APP.lib import mergeTrees

import numpy as NPY


def test():
    T1 = CGL.newCGNSTree()
    b1 = CGL.newCGNSBase(T1, "B1", 3, 3)
    b2 = CGL.newCGNSBase(T1, "B2", 3, 3)
    f1 = CGL.newFamily(b1, "F1")
    f2 = CGL.newFamily(b1, "F2")
    f1 = CGL.newFamily(b2, "F1")
    f2 = CGL.newFamily(b2, "F2")
    f3 = CGL.newFamily(b2, "F3")
    f4 = CGL.newFamily(b2, "F4")
    z1 = CGL.newZone(b1, "Z1", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z2 = CGL.newZone(b1, "Z2", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z3 = CGL.newZone(b1, "Z3", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z4 = CGL.newZone(b1, "Z4", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z5 = CGL.newZone(b1, "Z5", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z1 = CGL.newZone(b2, "Z1", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z2 = CGL.newZone(b2, "Z2", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z3 = CGL.newZone(b2, "Z3", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))

    T2 = CGL.newCGNSTree()
    b1 = CGL.newCGNSBase(T2, "B1", 3, 3)
    b2 = CGL.newCGNSBase(T2, "B2", 3, 3)
    b3 = CGL.newCGNSBase(T2, "B3", 2, 2)
    f1 = CGL.newFamily(b1, "F1")
    f1 = CGL.newFamily(b2, "F1")
    f2 = CGL.newFamily(b2, "F2")
    f3 = CGL.newFamily(b2, "F3")
    f4 = CGL.newFamily(b2, "F4")
    f5 = CGL.newFamily(b2, "F5")
    f1 = CGL.newFamily(b3, "F1")
    f2 = CGL.newFamily(b3, "F2")
    z1 = CGL.newZone(b1, "Z1", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z2 = CGL.newZone(b1, "Z2", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z3 = CGL.newZone(b1, "Z3", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z1 = CGL.newZone(b2, "Z1", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z2 = CGL.newZone(b2, "Z2", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z3 = CGL.newZone(b2, "Z3", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
    z4 = CGL.newZone(b2, "Z4", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))

    T3 = CGL.newCGNSTree()
    b1 = CGL.newCGNSBase(T3, "B1", 3, 3)
    b2 = CGL.newCGNSBase(T3, "B7", 3, 3)
    b3 = CGL.newCGNSBase(T3, "B8", 2, 2)
    f1 = CGL.newFamily(b1, "F1")
    f1 = CGL.newFamily(b2, "F8")

    Tlist = []
    for TT in [T1, T2, T3]:
        Tlist.append(TT)

    count = 0
    for t in Tlist:
        CGM.save("merge-T%.2d.hdf" % count, t, flags=CGM.S2P_TRACE)
        count += 1
    T = mergeTrees(Tlist)
    CGM.save("merge-Result.hdf", T, flags=CGM.S2P_TRACE)
    return T


# --- last line



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'CGNS/NAV/T/Q7MergeWindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Q7MergeWindow(object):
    def setupUi(self, Q7MergeWindow):
        Q7MergeWindow.setObjectName("Q7MergeWindow")
        Q7MergeWindow.resize(1124, 300)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/icons/cgSpy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Q7MergeWindow.setWindowIcon(icon)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Q7MergeWindow)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.bSelectA = QtWidgets.QPushButton(Q7MergeWindow)
        self.bSelectA.setMinimumSize(QtCore.QSize(24, 24))
        self.bSelectA.setMaximumSize(QtCore.QSize(24, 24))
        self.bSelectA.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/icons/user-A.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSelectA.setIcon(icon1)
        self.bSelectA.setCheckable(True)
        self.bSelectA.setChecked(True)
        self.bSelectA.setObjectName("bSelectA")
        self.horizontalLayout.addWidget(self.bSelectA)
        self.bSelectOrderSwap = QtWidgets.QPushButton(Q7MergeWindow)
        self.bSelectOrderSwap.setMinimumSize(QtCore.QSize(24, 24))
        self.bSelectOrderSwap.setMaximumSize(QtCore.QSize(24, 24))
        self.bSelectOrderSwap.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/images/icons/reverse.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSelectOrderSwap.setIcon(icon2)
        self.bSelectOrderSwap.setObjectName("bSelectOrderSwap")
        self.horizontalLayout.addWidget(self.bSelectOrderSwap)
        self.bSelectB = QtWidgets.QPushButton(Q7MergeWindow)
        self.bSelectB.setMinimumSize(QtCore.QSize(24, 24))
        self.bSelectB.setMaximumSize(QtCore.QSize(24, 24))
        self.bSelectB.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/images/icons/user-B.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSelectB.setIcon(icon3)
        self.bSelectB.setCheckable(True)
        self.bSelectB.setChecked(True)
        self.bSelectB.setObjectName("bSelectB")
        self.horizontalLayout.addWidget(self.bSelectB)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.bZoomOut = QtWidgets.QToolButton(Q7MergeWindow)
        self.bZoomOut.setMinimumSize(QtCore.QSize(25, 25))
        self.bZoomOut.setMaximumSize(QtCore.QSize(25, 25))
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/images/icons/level-out.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bZoomOut.setIcon(icon4)
        self.bZoomOut.setObjectName("bZoomOut")
        self.horizontalLayout.addWidget(self.bZoomOut)
        self.bZoomAll = QtWidgets.QPushButton(Q7MergeWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bZoomAll.sizePolicy().hasHeightForWidth())
        self.bZoomAll.setSizePolicy(sizePolicy)
        self.bZoomAll.setMinimumSize(QtCore.QSize(25, 25))
        self.bZoomAll.setMaximumSize(QtCore.QSize(25, 25))
        self.bZoomAll.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/images/icons/level-all.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bZoomAll.setIcon(icon5)
        self.bZoomAll.setObjectName("bZoomAll")
        self.horizontalLayout.addWidget(self.bZoomAll)
        self.bZoomIn = QtWidgets.QToolButton(Q7MergeWindow)
        self.bZoomIn.setMinimumSize(QtCore.QSize(25, 25))
        self.bZoomIn.setMaximumSize(QtCore.QSize(25, 25))
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/images/icons/level-in.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bZoomIn.setIcon(icon6)
        self.bZoomIn.setObjectName("bZoomIn")
        self.horizontalLayout.addWidget(self.bZoomIn)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.bSaveDiff = QtWidgets.QToolButton(Q7MergeWindow)
        self.bSaveDiff.setMinimumSize(QtCore.QSize(25, 25))
        self.bSaveDiff.setMaximumSize(QtCore.QSize(25, 25))
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/images/icons/select-save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSaveDiff.setIcon(icon7)
        self.bSaveDiff.setObjectName("bSaveDiff")
        self.horizontalLayout.addWidget(self.bSaveDiff)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.treeview = Q7DiffTreeView(Q7MergeWindow)
        self.treeview.viewport().setProperty("cursor", QtGui.QCursor(QtCore.Qt.CrossCursor))
        self.treeview.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.treeview.setAutoScroll(False)
        self.treeview.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.treeview.setProperty("showDropIndicator", False)
        self.treeview.setHorizontalScrollMode(QtWidgets.QAbstractItemView.ScrollPerItem)
        self.treeview.setIndentation(16)
        self.treeview.setRootIsDecorated(True)
        self.treeview.setUniformRowHeights(True)
        self.treeview.setExpandsOnDoubleClick(False)
        self.treeview.setObjectName("treeview")
        self.horizontalLayout_2.addWidget(self.treeview)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.bBackControl = QtWidgets.QPushButton(Q7MergeWindow)
        self.bBackControl.setMinimumSize(QtCore.QSize(25, 25))
        self.bBackControl.setMaximumSize(QtCore.QSize(25, 25))
        self.bBackControl.setText("")
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/images/icons/top.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bBackControl.setIcon(icon8)
        self.bBackControl.setObjectName("bBackControl")
        self.horizontalLayout_3.addWidget(self.bBackControl)
        self.bInfo = QtWidgets.QPushButton(Q7MergeWindow)
        self.bInfo.setMinimumSize(QtCore.QSize(25, 25))
        self.bInfo.setMaximumSize(QtCore.QSize(25, 25))
        self.bInfo.setText("")
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/images/icons/help-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bInfo.setIcon(icon9)
        self.bInfo.setObjectName("bInfo")
        self.horizontalLayout_3.addWidget(self.bInfo)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.bPreviousMark = QtWidgets.QToolButton(Q7MergeWindow)
        self.bPreviousMark.setMinimumSize(QtCore.QSize(25, 25))
        self.bPreviousMark.setMaximumSize(QtCore.QSize(25, 25))
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/images/icons/node-sids-opened.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bPreviousMark.setIcon(icon10)
        self.bPreviousMark.setObjectName("bPreviousMark")
        self.horizontalLayout_3.addWidget(self.bPreviousMark)
        self.bUnmarkAll_1 = QtWidgets.QToolButton(Q7MergeWindow)
        self.bUnmarkAll_1.setMinimumSize(QtCore.QSize(25, 25))
        self.bUnmarkAll_1.setMaximumSize(QtCore.QSize(25, 25))
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/images/icons/node-sids-leaf.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bUnmarkAll_1.setIcon(icon11)
        self.bUnmarkAll_1.setObjectName("bUnmarkAll_1")
        self.horizontalLayout_3.addWidget(self.bUnmarkAll_1)
        self.bNextMark = QtWidgets.QToolButton(Q7MergeWindow)
        self.bNextMark.setMinimumSize(QtCore.QSize(25, 25))
        self.bNextMark.setMaximumSize(QtCore.QSize(25, 25))
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/images/icons/node-sids-closed.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bNextMark.setIcon(icon12)
        self.bNextMark.setObjectName("bNextMark")
        self.horizontalLayout_3.addWidget(self.bNextMark)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem3)
        self.bClose = QtWidgets.QPushButton(Q7MergeWindow)
        self.bClose.setObjectName("bClose")
        self.horizontalLayout_3.addWidget(self.bClose)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)

        self.retranslateUi(Q7MergeWindow)
        QtCore.QMetaObject.connectSlotsByName(Q7MergeWindow)

    def retranslateUi(self, Q7MergeWindow):
        _translate = QtCore.QCoreApplication.translate
        Q7MergeWindow.setWindowTitle(_translate("Q7MergeWindow", "Form"))
        self.bSelectOrderSwap.setToolTip(_translate("Q7MergeWindow", "Reverse precedence order of merge"))
        self.bZoomOut.setToolTip(_translate("Q7MergeWindow", "Collapse lowest tree level"))
        self.bZoomOut.setText(_translate("Q7MergeWindow", "..."))
        self.bZoomAll.setToolTip(_translate("Q7MergeWindow", "Expand all tree"))
        self.bZoomIn.setToolTip(_translate("Q7MergeWindow", "Expand lowest tree level"))
        self.bZoomIn.setText(_translate("Q7MergeWindow", "..."))
        self.bSaveDiff.setToolTip(_translate("Q7MergeWindow", "Save tree view snapshot"))
        self.bSaveDiff.setText(_translate("Q7MergeWindow", "..."))
        self.bBackControl.setToolTip(_translate("Q7MergeWindow", "Raise Control window"))
        self.bInfo.setToolTip(_translate("Q7MergeWindow", "Contextual help"))
        self.bPreviousMark.setToolTip(_translate("Q7MergeWindow", "Select previous marked node"))
        self.bPreviousMark.setText(_translate("Q7MergeWindow", "..."))
        self.bUnmarkAll_1.setToolTip(_translate("Q7MergeWindow", "Unmark all nodes"))
        self.bUnmarkAll_1.setText(_translate("Q7MergeWindow", "..."))
        self.bNextMark.setToolTip(_translate("Q7MergeWindow", "Select next marked node"))
        self.bNextMark.setText(_translate("Q7MergeWindow", "..."))
        self.bClose.setText(_translate("Q7MergeWindow", "Close"))
from CGNS.NAV.mdifftreeview import Q7DiffTreeView
from . import Res_rc



#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = "gridconnectivity1to1"
diag = True
vertexsize = 20
cellsize = 7
ntris = 12


def makeCorrectTree():
    T = CGL.newCGNSTree()
    b = CGL.newBase(T, "Base", 3, 3)
    z1 = CGL.newZone(
        b, "Zone1", NPY.array([[5, 4, 0], [7, 6, 0], [5, 4, 0]], order="F")
    )
    g = CGL.newGridCoordinates(z1, "GridCoordinates")
    d = CGL.newDataArray(
        g, CGK.CoordinateX_s, NPY.ones((5, 7, 5), dtype="float64", order="F")
    )
    d = CGL.newDataArray(
        g, CGK.CoordinateY_s, NPY.ones((5, 7, 5), dtype="float64", order="F")
    )
    d = CGL.newDataArray(
        g, CGK.CoordinateZ_s, NPY.ones((5, 7, 5), dtype="float64", order="F")
    )
    s = NPY.array([[vertexsize, cellsize, 0]], dtype="int32", order="F")
    z2 = CGU.copyNode(z1, "Zone2")
    b[2].append(z2)
    z = [z1, z2]
    return (T, b, z)


(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join1_2",
    "Zone2",
    NPY.array([[1, 1], [1, 7], [1, 5]]),
    NPY.array([[1, 5], [1, 7], [5, 5]]),
    NPY.array([+1, +2, +3]),
)
zgc = CGL.newZoneGridConnectivity(z[1])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join2_1",
    "Zone1",
    NPY.array([[1, 5], [1, 7], [5, 5]]),
    NPY.array([[1, 1], [1, 7], [1, 5]]),
    NPY.array([+1, +2, +3]),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "gridconnectivity1to1 bad datatype"
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join1_2",
    "Zone2",
    NPY.array([[1, 1], [1, 7], [1, 5]]),
    NPY.array([[1, 1], [1, 7], [5, 5]]),
    NPY.array([+1, +2, +3]),
)
gc[1] = NPY.array([1], order="F")
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "gridconnectivity1to1 absent opposite GridConnectivity"
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join1_2",
    "Zone2",
    NPY.array([[1, 1], [1, 7], [1, 5]]),
    NPY.array([[1, 5], [1, 7], [5, 5]]),
    NPY.array([+1, +2, +3]),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "gridconnectivity1to1 unvalid opposite GridConnectivity"
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join1_2",
    "Zone2",
    NPY.array([[1, 1], [1, 7], [1, 5]]),
    NPY.array([[1, 5], [1, 7], [5, 5]]),
    NPY.array([+1, +2, +3]),
)
zgc = CGL.newZoneGridConnectivity(z[1])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join2_1",
    "Zone1",
    NPY.array([[1, 5], [1, 7], [5, 5]]),
    NPY.array([[1, 1], [1, 7], [1, 5]]),
    NPY.array([+1, +2, +3]),
)
CGU.removeChildByName(gc, CGK.PointRangeDonor_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "gridconnectivity1to1 bad shape on Transform"
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join1_2",
    "Zone2",
    NPY.array([[1, 1], [1, 7], [1, 5]]),
    NPY.array([[1, 5], [1, 7], [5, 5]]),
    NPY.array([[+1, +2, +3]]),
)
zgc = CGL.newZoneGridConnectivity(z[1])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join2_1",
    "Zone1",
    NPY.array([[1, 5], [1, 7], [5, 5]]),
    NPY.array([[1, 1], [1, 7], [1, 5]]),
    NPY.array([+1, +2, +3]),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "gridconnectivity1to1 bad Transform"
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join1_2",
    "Zone2",
    NPY.array([[1, 1], [1, 7], [1, 5]]),
    NPY.array([[1, 5], [1, 7], [5, 5]]),
    NPY.array([+1, +2, -4]),
)
zgc = CGL.newZoneGridConnectivity(z[1])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join2_1",
    "Zone1",
    NPY.array([[1, 5], [1, 7], [5, 5]]),
    NPY.array([[1, 1], [1, 7], [1, 5]]),
    NPY.array([+1, +2, +4]),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "gridconnectivity1to1 several 0 in Transform"
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join1_2",
    "Zone2",
    NPY.array([[1, 1], [1, 7], [1, 5]]),
    NPY.array([[1, 5], [1, 7], [5, 5]]),
    NPY.array([+1, 0, 0]),
)
zgc = CGL.newZoneGridConnectivity(z[1])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join2_1",
    "Zone1",
    NPY.array([[1, 5], [1, 7], [5, 5]]),
    NPY.array([[1, 1], [1, 7], [1, 5]]),
    NPY.array([+1, +2, +3]),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "gridconnectivity1to1 not a face"
diag = False
(T, b, z) = makeCorrectTree()
zgc = CGL.newZoneGridConnectivity(z[0])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join1_2",
    "Zone2",
    NPY.array([[1, 5], [1, 7], [1, 5]]),
    NPY.array([[1, 5], [1, 7], [1, 5]]),
    NPY.array([+1, +2, +3]),
)
zgc = CGL.newZoneGridConnectivity(z[1])
gc = CGL.newGridConnectivity1to1(
    zgc,
    "join2_1",
    "Zone1",
    NPY.array([[1, 5], [1, 7], [1, 5]]),
    NPY.array([[1, 5], [1, 7], [1, 5]]),
    NPY.array([+1, +2, +3]),
)
TESTS.append((tag, T, diag))



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  ---------------------------------------------------------------------------
#
from . import CGNSBase_t
from . import Zone_t
from . import ZoneBC_t
from . import BC_t
from . import DataClass_t
from . import Descriptor_t
from . import DimensionalUnits_t
from . import DimensionalExponents_t
from . import GridLocation_t
from . import PointList_t
from . import PointRange_t
from . import Rind_t
from . import DataConversion_t
from . import SimulationType_t
from . import Ordinal_t
from . import GridCoordinates_t
from . import DataArray_t
from . import DiscreteData_t
from . import Elements_t
from . import BCDataSet_t
from . import BCData_t
from . import BCProperty_t
from . import RotatingCoordinates_t
from . import ZoneGridConnectivity_t
from . import GridConnectivity1to1_t
from . import GridConnectivity_t
from . import GridConnectivityProperty_t
from . import AverageInterface_t
from . import OversetHoles_t
from . import FlowSolution_t
from . import FlowEquationSet_t
from . import GoverningEquations_t
from . import GasModel_t
from . import ThermalConductivityModel_t
from . import ThermalRelaxationModel_t
from . import ChemicalKineticModel_t
from . import EMElectricFieldModel_t
from . import EMMagneticFieldModel_t
from . import EMConductivityModel_t
from . import ViscosityModel_t
from . import TurbulenceClosure_t
from . import TurbulenceModel_t
from . import AxiSymmetry_t
from . import BaseIterativeData_t
from . import ZoneIterativeData_t
from . import RigidGridMotion_t
from . import ArbitraryGridMotion_t
from . import ReferenceState_t
from . import ConvergenceHistory_t
from . import IntegralData_t
from . import UserDefinedData_t
from . import Gravity_t
from . import Family_t
from . import FamilyName_t
from . import FamilyBC_t
from . import GeometryReference_t
from . import ZoneSubRegion_t

profile = {
    "CGNSBase_t": CGNSBase_t.pattern,
    "Zone_t": Zone_t.pattern,
    "ZoneBC_t": ZoneBC_t.pattern,
    "BC_t": BC_t.pattern,
    "DataClass_t": DataClass_t.pattern,
    "Descriptor_t": Descriptor_t.pattern,
    "DimensionalUnits_t": DimensionalUnits_t.pattern,
    "DimensionalExponents_t": DimensionalExponents_t.pattern,
    "GridLocation_t": GridLocation_t.pattern,
    "PointList_t": PointList_t.pattern,
    "PointRange_t": PointRange_t.pattern,
    "Rind_t": Rind_t.pattern,
    "DataConversion_t": DataConversion_t.pattern,
    "SimulationType_t": SimulationType_t.pattern,
    "Ordinal_t": Ordinal_t.pattern,
    "GridCoordinates_t": GridCoordinates_t.pattern,
    "DataArray_t": DataArray_t.pattern,
    "DiscreteData_t": DiscreteData_t.pattern,
    "Elements_t": Elements_t.pattern,
    "BCDataSet_t": BCDataSet_t.pattern,
    "BCData_t": BCData_t.pattern,
    "BCProperty_t": BCProperty_t.pattern,
    "RotatingCoordinates_t": RotatingCoordinates_t.pattern,
    "ZoneGridConnectivity_t": ZoneGridConnectivity_t.pattern,
    "GridConnectivity1to1_t": GridConnectivity1to1_t.pattern,
    "GridConnectivity_t": GridConnectivity_t.pattern,
    "GridConnectivityProperty_t": GridConnectivityProperty_t.pattern,
    "AverageInterface_t": AverageInterface_t.pattern,
    "OversetHoles_t": OversetHoles_t.pattern,
    "FlowSolution_t": FlowSolution_t.pattern,
    "FlowEquationSet_t": FlowEquationSet_t.pattern,
    "GoverningEquations_t": GoverningEquations_t.pattern,
    "GasModel_t": GasModel_t.pattern,
    "ThermalConductivityModel_t": ThermalConductivityModel_t.pattern,
    "ThermalRelaxationModel_t": ThermalRelaxationModel_t.pattern,
    "ChemicalKineticModel_t": ChemicalKineticModel_t.pattern,
    "EMElectricFieldModel_t": EMElectricFieldModel_t.pattern,
    "EMMagneticFieldModel_t": EMMagneticFieldModel_t.pattern,
    "EMConductivityModel_t": EMConductivityModel_t.pattern,
    "ViscosityModel_t": ViscosityModel_t.pattern,
    "TurbulenceClosure_t": TurbulenceClosure_t.pattern,
    "TurbulenceModel_t": TurbulenceModel_t.pattern,
    "AxiSymmetry_t": AxiSymmetry_t.pattern,
    "BaseIterativeData_t": BaseIterativeData_t.pattern,
    "ZoneIterativeData_t": ZoneIterativeData_t.pattern,
    "RigidGridMotion_t": RigidGridMotion_t.pattern,
    "ArbitraryGridMotion_t": ArbitraryGridMotion_t.pattern,
    "ReferenceState_t": ReferenceState_t.pattern,
    "ConvergenceHistory_t": ConvergenceHistory_t.pattern,
    "IntegralData_t": IntegralData_t.pattern,
    "UserDefinedData_t": UserDefinedData_t.pattern,
    "Gravity_t": Gravity_t.pattern,
    "Family_t": Family_t.pattern,
    "FamilyName_t": FamilyName_t.pattern,
    "FamilyBC_t": FamilyBC_t.pattern,
    "GeometryReference_t": GeometryReference_t.pattern,
    "ZoneSubRegion_t": ZoneSubRegion_t.pattern,
}
#



#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import CGNS.PAT.cgnslib        as CGL
cimport numpy as NCY
import numpy as NPY
from cpython cimport bool as py_bool 

def checkReserved(node,childtype,childname,pth,log):
    childnode=CGU.hasChildName(node,childname)
    if (childnode is None): return True
    if (childnode[3]!=childtype):
        log.push(pth,'S120',childname,childtype)
        return False
    return True

def checkChildValue(tnode,tshape,tdtype,pth,log,rs,tmin=None,tmax=None):
   if (tnode is not None):
       if (CGU.getShape(tnode)!=tshape):
           rs=log.push(pth,'S192',CGU.getShape(tnode),tshape,tnode[0])
       elif (CGU.getValueDataType(tnode) not in tdtype):
           rs=log.push(pth,'S196',tnode[0])
       elif ((tmin is not None) and (tnode[1][0] < tmin)):
           rs=log.push(pth,'S197',tnode[0])
       elif ((tmax is not None) and (tnode[1][0] > tmax)):
           rs=log.push(pth,'S197',tnode[0])
   return rs

def getBase(pth,node,parent,tree,log):
    lpth=CGU.getPathToList(pth,True)
    if (lpth==[]): return None
    if (lpth==[CGK.CGNSTree_ts]): return None
    if (lpth[0]!=[CGK.CGNSTree_ts]): return lpth[0]
    return lpth[1]
  
cpdef py_bool indexInRangeList(int idx,rl):
  """return True if index in range index list rl 
    - Args:
     * `var`: integer
     * `rl`: list of range index [[i1min,i1max],[i2min,i2max],..]
  """
  for r in rl:
    if (idx>=r[0] and idx<=r[1]):
      return True
  return False

cpdef int countIdxInRangeList(int idx,rl):
  """return the number of times index appears in range index list rl 
    - Args:
     * `var`: integer
     * `rl`: list of range index [[i1min,i1max],[i2min,i2max],..]
  """
  cdef int n
  n = 0
  for r in rl:
    if (idx>=r[0] and idx<=r[1]): n += 1
  return n

def allIndexInElementRangeList(NCY.ndarray t,erl,pth,log,rs):
  cdef int idx
  cdef py_bool test

  for idx in t:
    test = False
    for et in erl:
      if (indexInRangeList(idx,erl[et])): 
        test=True
        break
    if (not test): 
      rs=log.push(pth,'S206')
      return rs
  return rs

# -----------------------------------------------------------------------------
def getElementTypes(level,physicaldimension):
  """Element types of certain level for certain physical dimension::

    # The level may be Cell, Face, Edge or Vertex

    # The physical dimension is within 0-3
  
  - Args:
   * `level`: may be Cell, Face, Edge or Vertex (`string`)
   * `physicaldimension`: physical dimension of CFD problem (`int`)

  - Return:
   * The list of Element Types of considered level for considered physical dimension
  """
  if   (level == CGK.Cell_s and physicaldimension == 3):
    return CGK.ElementType3D+[CGK.NFACE_n]
  elif (level == CGK.Cell_s and physicaldimension == 2):
    return CGK.ElementType2D+[CGK.NFACE_n]
  elif (level == CGK.Cell_s and physicaldimension == 1):
    return CGK.ElementType1D+[CGK.NFACE_n]
    
  elif (level == CGK.Face_s and physicaldimension == 3):
    return CGK.ElementType2D+[CGK.NGON_n]    
  elif (level == CGK.Face_s and physicaldimension == 2):
    return CGK.ElementType1D+[CGK.NGON_n] 
  elif (level == CGK.Face_s and physicaldimension == 1):
    return CGK.ElementType0D+[CGK.NGON_n] 

  elif (level == CGK.Edge_s and physicaldimension == 3):
    return CGK.ElementType1D
  elif (level == CGK.Edge_s and physicaldimension == 2):
    return CGK.ElementType0D
  
  elif (level == CGK.Vertex_s and physicaldimension == 3):
    return CGK.ElementType0D 
        
  else :
    return []  

# -----------------------------------------------------------------------------
def getAuthElementTypes(facetype):
  """Authorized Element types for a Face type::
  
  - Args:
   * `facetype`: Face Type (`int`)

  - Return:
   * The list of Element Types authorized for the considered face type
  """  
  if (facetype in CGK.ElementType2D):
    if (facetype in CGK.ElementType_tri):
      return CGK.ElementType_trionly+CGK.ElementType_triquad+[CGK.MIXED,CGK.NFACE_n]
    elif (facetype in CGK.ElementType_quad):
      return CGK.ElementType_quadonly+CGK.ElementType_triquad+[CGK.MIXED,CGK.NFACE_n]
    else:
      return CGK.ElementType3D+[CGK.NFACE_n]
  elif (facetype==CGK.NGON_n):
    return CGK.ElementType3D+[CGK.NFACE_n]
  else:
    return []

# -----------------------------------------------------------------------------
def getAuthGridLocation(celldim,regioncelldim):
  """Authorized GridLocation wrt celldim and regioncelldim::
  
  - Args:
   * `celldim`: Cell dimension (`int`)
   * `regioncelldim`: Region cell dimension (`int`)

  - Return:
   * The list of GridLocation authorized for celldim/regioncelldim
  """  
  if (celldim == 3):
    if (regioncelldim == 3):
      return [CGK.Vertex_s,
              CGK.EdgeCenter_s,
              CGK.FaceCenter_s,CGK.IFaceCenter_s,CGK.JFaceCenter_s,CGK.KFaceCenter_s,
              CGK.CellCenter_s]
    elif (regioncelldim == 2):
      return [CGK.Vertex_s,
              CGK.EdgeCenter_s,
              CGK.FaceCenter_s,CGK.IFaceCenter_s,CGK.JFaceCenter_s,CGK.KFaceCenter_s]
    elif (regioncelldim == 1):
      return [CGK.Vertex_s,
              CGK.EdgeCenter_s]
    elif (regioncelldim == 0):
      return [CGK.Vertex_s]    
  elif (celldim == 2):
    if (regioncelldim == 2):
      return [CGK.Vertex_s,
              CGK.EdgeCenter_s,
              CGK.CellCenter_s]
    elif (regioncelldim == 1):
      return [CGK.Vertex_s,
              CGK.EdgeCenter_s]    
    elif (regioncelldim == 0):
      return [CGK.Vertex_s]    
  elif (celldim == 1):
    if (regioncelldim == 1):
      return [CGK.Vertex_s,
              CGK.CellCenter_s]
    elif (regioncelldim == 0):
      return [CGK.Vertex_s]          
  return []

def getGridLocation(node,pth=None,log=None,rs=None):
  return getChildNodeValueWithDefault(node,CGK.GridLocation_ts,CGK.Vertex_s,
                                      pth=pth,log=log,rs=rs)

def getGridConnectivityType(node,pth=None,log=None,rs=None):
  return getChildNodeValueWithDefault(node,CGK.GridConnectivityType_ts,CGK.Overset_s,
                                      pth=pth,log=log,rs=rs)
  
def getChildNodeValueWithDefault(node,ntype,defaultv,pth=None,log=None,rs=None):
  p = CGU.hasChildType(node,ntype) # get child by type
  if (not p): # child is absent
    v=defaultv # affect default value
    if rs: rs=log.push(pth,'s0000.0123',ntype,defaultv)
  else: v=p[0][1].tostring() # child is present
  if rs: return v,rs
  else: return v  

def listLength(node):
  """the structure function ListLength is used to specify the number of entities (e.g. vertices) 
     corresponding to a given PointRange or PointList. 
     If PointRange is specified, then ListLength is obtained from the number of points (inclusive)
     between the beginning and ending indices of PointRange. 
     If PointList is specified, then ListLength is the number of indices in the list of points. 
     In this situation, ListLength becomes a user input along with the indices of the list PointList. 
     By user we mean the application code that is generating the CGNS database::
     
  - Args:
   * `node`: cgns node, should be a node of type PointList or PointRange

  - Return:
   * ListLength, number of elements as defined in PointList/PointRange
  """
  CGU.checkNode(node)
  listlength = -1
  if (node[0] == CGK.PointList_s):
    listlength = node[1].size
  elif (node[0] == CGK.PointRange_s):
    shp=CGU.getShape(node)
    if (len(shp)==2):
      if (    shp[0] in range(1,3+1) 
          and shp[1]==2
          and CGU.getValueDataType(node) in [CGK.I4,CGK.I8]):
        pr = node[1]
        listlength = 0        
        for d in range(shp[0]):
          listlength += max(pr[d])-min(pr[d])+1
  return listlength

# -----------------------------------------------------------------------------
def getListLength(node):
  """return ListLength, number of elements as defined in PointList/PointRange node or child::
  
  - Args:
   * `node`: cgns node, should have a child or be itself a node of type PointList or PointRange

  - Return:
   * ListLength, number of elements as defined in PointList/PointRange
  """
  listlength = -1
  if (node[0] in [CGK.PointList_s,CGK.PointRange_s]):
    listlength = listLength(node)
  else:
    if (CGU.hasChildName(node,CGK.PointList_s)):
      listlength = listLength(CGU.hasChildName(node,CGK.PointList_s))
    if (CGU.hasChildName(node,CGK.PointRange_s)):
      listlength = listLength(CGU.hasChildName(node,CGK.PointRange_s))
  return listlength

def dataSize(node,indexdimension,vertexsize,cellsize):
  """The function DataSize[] is the size of flow solution data arrays. 
     If Rind is absent then DataSize represents only the core points; 
     it will be the same as VertexSize or CellSize depending on GridLocation.::
     
  - Args:
   * `node`: cgns node

  - Return:
   * DataSize, one-dimensional int array of length IndexDimension 
  """
  gridloc=getGridLocation(node)
  datasize=tuple([])
  if (CGU.hasChildName(node,CGK.PointList_s) or CGU.hasChildName(node,CGK.PointRange_s)):
    datasize = tuple([getListLength(node)])
  elif (not CGU.hasChildType(node,CGK.Rind_ts)):
    if (gridloc == CGK.Vertex_s):
      datasize = vertexsize
    elif (gridloc == CGK.CellCenter_s):
      datasize = cellsize
  elif (CGU.hasChildType(node,CGK.Rind_ts)):
    rind=CGU.hasChildType(node,CGK.Rind_ts)[0]
    if (    CGU.getShape(rind)==(2*indexdimension,)
        and CGU.getValueDataType(rind) in [CGK.I4,CGK.I8]):
      datasize=[]
      if (gridloc == CGK.Vertex_s):
        for d in range(indexdimension):
          datasize.append( vertexsize[d]
                          +rind[1][2*d]+rind[1][2*d+1])
      elif (gridloc == CGK.CellCenter_s):
        for d in range(indexdimension):
          datasize.append( cellsize[d]
                          +rind[1][2*d]+rind[1][2*d+1])
      datasize=tuple(datasize)
  return datasize

def getZoneType(tree,path):
  """Return type Structured|Unstructured of zone in path::
     
  - Args:
   * `tree`: cgns tree
   * `path`: node path in tree

  - Return:
   * Structured|Unstructured
  """
  l=CGU.getPathAsTypes(tree,path)
  for (i,var) in enumerate(l):
    if (var == CGK.Zone_ts): break
  if (i==len(l)): return None
  else:
    zpth=CGU.getPathAncestor(path,level=len(l)-i-1)
    zone=CGU.getNodeByPath(tree,zpth)
    zt=CGU.hasChildName(zone,CGK.ZoneType_s)
    if (zt is not None):
      return zt[1].tostring()
    else:
      return None

def getZoneName(tree,path):
  """Return name of zone ::
     
  - Args:
   * `tree`: cgns tree
   * `path`: node path in tree

  - Return:
   * zone name
  """
  name=None
  l=CGU.getPathAsTypes(tree,path)
  for (i,var) in enumerate(l):
    if (var == CGK.Zone_ts): break
  if (i!=len(l)):
    zpth=CGU.getPathAncestor(path,level=len(l)-i-1)
    zone=CGU.getNodeByPath(tree,zpth)
    name=zone[0]
  return name

def getLevelFromGridLoc(gridloc):
  if (gridloc.endswith(CGK.FaceCenter_s)):
    level = CGK.Face_s
  elif (gridloc == CGK.CellCenter_s):
    level = CGK.Cell_s
  elif (gridloc == CGK.Vertex_s):
    level = CGK.Vertex_s
  elif (gridloc == CGK.EdgeCenter_s):
    level = CGK.Edge_s
  else:
    level = None
  return level

def getElementTypeRangeList(level,physicaldimension,elementrangelist):
  """extract from elementrangelist the range list corresponding to Element 
     types of certain level for certain physical dimension::

    # The level may be Cell, Face, Edge or Vertex
    # The physical dimension is within 0-3
  
  - Args:
   * `level`: may be Cell, Face, Edge or Vertex (`string`)
   * `physicaldimension`: physical dimension of CFD problem (`int`)
   * `elementrangelist`: dictionary element type <-> list of ranges

  - Return:
   * a dictionary element type of the considered level <-> list of ranges
  """
  etypes = getElementTypes(level,physicaldimension)
  etyperl = elementrangelist.copy()
  for k in elementrangelist:
    if k not in etypes: del etyperl[k]
  return etyperl

def getIndices(tree,zonepath,nodetype,indextype,indexdimension):
  """look for all indices of indextype for all nodes of type nodetype in zone
     defined through its path in tree::

    # typical nodetype: BC_t or ZoneGridConnectivity
  
  - Args:
   * `tree`: tree
   * `zonepath`: zone path in tree
   * `nodetype`: node type to look for index in
   * `indextype`: IndexRange or IndexArray
   * `indexdimension`: IndexDimension of zone

  - Return:
   * a list of ranges or arrays, depending on indextype
  """
  pth = []
  if (nodetype in [CGK.ZoneBC_ts,CGK.BC_ts]):
    #pth = CGU.getAllNodesByTypeSet(tree,typeset)
    for child in CGU.getAuthChildren(CGL.newZoneBC(None)):
      searchpath=CGU.getPathToList(zonepath,True)+[CGK.ZoneBC_ts]+[child[0]]
      pth += CGU.getAllNodesByTypeOrNameList(tree,[CGK.CGNSTree_ts]+searchpath+[indextype])
  elif (nodetype in [CGK.ZoneGridConnectivity_ts,CGK.GridConnectivity_ts,CGK.GridConnectivity1to1_ts]):
    for child in CGU.getAuthChildren(CGL.newZoneGridConnectivity(None)):
      searchpath=CGU.getPathToList(zonepath,True)+[CGK.ZoneGridConnectivity_ts]+[child[0]]
      # cannot consider indextype since should not consider Point[Range/List]Donor
      if (indextype == CGK.IndexRange_ts):
        pth += CGU.getAllNodesByTypeOrNameList(tree,[CGK.CGNSTree_ts]+searchpath+[CGK.PointRange_s])
        pth += CGU.getAllNodesByTypeOrNameList(tree,[CGK.CGNSTree_ts]+searchpath+[CGK.ElementRange_s])
      elif (indextype == CGK.IndexArray_ts):
        pth += CGU.getAllNodesByTypeOrNameList(tree,[CGK.CGNSTree_ts]+searchpath+[CGK.PointList_s])
  pth = [path for path in pth if path]
  if (indextype not in [CGK.IndexRange_ts,CGK.IndexArray_ts]): return []
  if (indextype == CGK.IndexRange_ts):
    rl = []
    if (pth):
      for path in pth:
        node = CGU.getNodeByPath(tree,path)
        if (CGU.getShape(node) == (indexdimension,2)):
          rl.append(node[1])
    return rl
  elif (indextype == CGK.IndexArray_ts):
    al = NPY.empty((indexdimension,0),dtype=NPY.int32)
    if (pth):  
      for path in pth:
        node = CGU.getNodeByPath(tree,path)
        if (CGU.getShape(node)[0] == indexdimension):
          al = NPY.concatenate([al,node[1]],axis=1)
    return al  

def getIndicesOnBCandGC(tree,zonepath,indexdim):
  """look for all IndexRange and IndexArray on BC and GridConnectivity of a zone
  
  - Args:
   * `tree`: tree
   * `zonepath`: zone path in tree

  - Return:
   * a list of ranges and a list of arrays
  """
  rl = []
  al = NPY.empty((indexdim,0),dtype=NPY.int32)
  rl += getIndices(tree,zonepath,CGK.ZoneBC_ts,CGK.IndexRange_ts,indexdim)
  rl += getIndices(tree,zonepath,CGK.ZoneGridConnectivity_ts,CGK.IndexRange_ts,indexdim)
  al = NPY.concatenate([al,getIndices(tree,zonepath,CGK.ZoneBC_ts,CGK.IndexArray_ts,indexdim)],axis=1)
  al = NPY.concatenate([al,getIndices(tree,zonepath,CGK.ZoneGridConnectivity_ts,CGK.IndexArray_ts,indexdim)],axis=1)
  return (rl,al) 

def getFaceNumber(facerange,zd):
  """return an integer within [0..celldimension*2-1] corresponding to the face of facerange,
     0 <-> imin
     1 <-> imax
     ...
     5 <-> kmax
     return also the face indexes
     Note: this face indexing is not coherent with CGNS SIDS where, on 3D hexaedra:
     1 <-> kmin
     2 <-> jmin
     3 <-> imax
     4 <-> jmax
     5 <-> imin
     6 <-> kmax
     but the last is not coherent with CGNS SIDs edge indexing, where, on 2D quad:
     1 <-> jmin
     2 <-> imax
     3 <-> jmax
     4 <-> imin 
  - Args:
   * `facerange`: IndexRange of the face
   * `zd`: zone node value

  - Return:
   * five integers corresponding to the face of facerange and its indices
  """
  face = -1
  if (    facerange.ndim == 2
      and facerange.shape[1] == 2
      and zd.ndim == 2
      and facerange.shape[0] == zd.shape[0]):
    indexdimension = facerange.shape[0]
    for idx in range(indexdimension):
      if (    facerange[idx,0]==facerange[idx,1]
          and face == -1):
        if (facerange[idx,0] == 1):
          face = 2*idx
        elif (facerange[idx,0] == zd[idx,0]):
          face = 2*idx+1          
      elif (    facerange[idx,0]==facerange[idx,1]
            and face != -1): # this is not a face
        face = -1
        break
    if (face in [0,1]):
      idx_i = 1
      idx_j = 2
    elif (face in [2,3]):
      idx_i = 0
      idx_j = 2    
    elif (face in [4,5]):
      idx_i = 0
      idx_j = 1
  if (face != -1):
    if (indexdimension == 1):
      return (face,1,2,1,2)
    elif (indexdimension == 2):
      return (face,min(facerange[idx_i]),max(facerange[idx_i]),1,2)
    elif (indexdimension == 3):
      return (face,min(facerange[idx_i]),max(facerange[idx_i]),min(facerange[idx_j]),max(facerange[idx_j]))
    else:
      return (face,0,0,0,0)
  else:
    return (face,0,0,0,0)

def bdnName(bnd):
  if   (bnd==0): bnd_name='imin'
  elif (bnd==1): bnd_name='imax'
  elif (bnd==2): bnd_name='jmin'
  elif (bnd==3): bnd_name='jmax'
  elif (bnd==4): bnd_name='kmin'
  elif (bnd==5): bnd_name='kmax'
  else: bnd_name=''
  return bnd_name

def initBndFaces(zd):
  """return NPY.zeros of sizes the faces of a structured zone, face centered
  
  - Args:
   * `zd`: zone dimension (zone node value)

  - Return:
   * a list of NPY.zeros of sizes the faces of the structured zone, face centered
  """
  celldimension=zd.shape[0]
  if (celldimension == 1):
    bndfaces=NPY.zeros([1,1],dtype=NPY.int32),\
             NPY.zeros([1,1],dtype=NPY.int32)
  elif (celldimension == 2):
    bndfaces=NPY.zeros([zd[1,1],1],dtype=NPY.int32),\
             NPY.zeros([zd[1,1],1],dtype=NPY.int32),\
             NPY.zeros([zd[0,1],1],dtype=NPY.int32),\
             NPY.zeros([zd[0,1],1],dtype=NPY.int32)
  elif (celldimension == 3):
    bndfaces=NPY.zeros([zd[1,1],zd[2,1]],dtype=NPY.int32),\
             NPY.zeros([zd[1,1],zd[2,1]],dtype=NPY.int32),\
             NPY.zeros([zd[0,1],zd[2,1]],dtype=NPY.int32),\
             NPY.zeros([zd[0,1],zd[2,1]],dtype=NPY.int32),\
             NPY.zeros([zd[0,1],zd[1,1]],dtype=NPY.int32),\
             NPY.zeros([zd[0,1],zd[1,1]],dtype=NPY.int32)
  else:
    bndfaces=NPY.empty(0)
  return bndfaces

def isAFace(pr):
  """return True if pointrange is a face
  
  - Args:
   * `pr`: IndexRange node value

  - Return:
   * True if pointrange is a face
  """
  answer = False
  if (pr.ndim == 2 and pr.shape[1] == 2):
    for d in range(pr.shape[0]):
      if (pr[d][0] == pr[d][1]):
        answer = True
        break 
  return answer
                     
def hasOneAndOnlyOneChildAmong(node,namelist):
  """return True if node has one and only one child with name among listed
  
  - Args:
   * `node`: node
   * `namelist`: list of names

  - Return:
   * True if node has one and only one child with name among listed
  """
  count = 0
  for name in namelist:
    if (CGU.hasChildName(node,name)): count += 1
  if (count != 1):
    return False
  else:
    return True




cdef extern from "hashutils.h":
  ctypedef struct __f_entry_t:
      pass
  
  ctypedef __f_entry_t* F_entry_ptr
  
  F_entry_ptr newHashTable(int s) nogil
  
  void freeHashTable(F_entry_ptr,int) nogil
  void addHashEntry(F_entry_ptr tab,int tsz,
                    int p1,int p2,int p3,int p4,int idx,int sec) nogil
  int  fetchHashEntry(F_entry_ptr tab,int tsz,
                      int p1,int p2,int p3,int p4,int sec) nogil
  int exteriorHashEntry(F_entry_ptr tab,int,int,int,int,int,int) nogil
